-- ------------------------------------------------------------------------------ --
--                           TradeSkillMaster_Vendoring                           --
--           http://www.curse.com/addons/wow/tradeskillmaster_vendoring           --
--                                                                                --
--             A TradeSkillMaster Addon (http://tradeskillmaster.com)             --
--    All Rights Reserved* - Detailed license information included with addon.    --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local Buy = TSM:NewModule("Buy", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendoring") -- loads the localization table

local private = { quantity = 1 }

function Buy:OnEnable()

end

function Buy:CreateTab(parent)

	local frame = CreateFrame("Frame", nil, parent)
	frame:Hide()
	frame:SetAllPoints()
	frame:SetScript("OnShow", function() private.Show() end)
		
	local stContainer = CreateFrame("Frame", nil, frame)
	stContainer:SetPoint("TOPLEFT", 5, -45)
	stContainer:SetPoint("BOTTOMRIGHT", -5, 75)
	TSMAPI.Design:SetFrameColor(stContainer)
	
	local stCols = {
		{
			name = L["Item"],
			width = 0.4,
		},
		{
			name = L["Cost"],
			width = 0.4
		},
		{
			name = L["Quantity"],
			width = 0.2		
		}
	}		
	
	local handlers = {
		OnClick = function(_, data, self, button)
			if not data then return end
											
			if button == "LeftButton" then							
				if IsShiftKeyDown() then
				   BuyMerchantItem(data.index,data.stackCount)
				end										
			end
		end,
		OnEnter = function(_, data, self)
			if not data then return end
			local color = TSMAPI.Design:GetInlineColor("link")
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			TSMAPI:SafeTooltipLink(data.itemLink)
			GameTooltip:Show()
		end,
		OnLeave = function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
			BattlePetTooltip:Hide()
		end
	}

	local st = TSMAPI:CreateScrollingTable(stContainer, stCols, handlers)
	st:SetData({})
	frame.st = st
	
	local itemQuantityLabel = TSMAPI.GUI:CreateLabel(frame, "small")
	itemQuantityLabel:SetPoint("BOTTOMLEFT", 5, 25)
	itemQuantityLabel:SetHeight(20)
	itemQuantityLabel:SetJustifyV("CENTER")
	itemQuantityLabel:SetJustifyH("LEFT")
	itemQuantityLabel:SetText(L["Quantity"])

	local qtyBox = TSMAPI.GUI:CreateInputBox(frame)
	qtyBox:SetPoint("TOPLEFT", itemQuantityLabel, "TOPRIGHT", 5, 0)
	qtyBox:SetPoint("TOPRIGHT", -95, -65)
	qtyBox:SetHeight(20)
	qtyBox:SetNumeric(true)
	qtyBox:SetNumber(private.quantity)	
	qtyBox:SetScript("OnEnterPressed", function(self)
			self:ClearFocus()
		end)
	qtyBox:SetScript("OnEditFocusLost", function(self)
			self:HighlightText(0, 0)
			private.quantity = self:GetNumber()
		end)	
	frame.qtyBox = qtyBox
	
	local btn = TSMAPI.GUI:CreateButton(frame, 18)	
	btn:SetPoint("BOTTOMLEFT", qtyBox, "BOTTOMRIGHT", 5, 0)
	btn:SetWidth(50)
	btn:SetHeight(20)
	btn:SetText(L["Buy"])
	btn:SetScript("OnClick", function() private:Buy() end)
	frame.buyBtn = btn	
	
	private.frame = frame
	return frame
end

function private:Buy()
	if private.frame and private.frame.st then	
		
		local selection = private.frame.st:GetSelection()
		if selection and private.quantity > 0 then		
			local row = private.frame.st.rowData[selection]
		
			BuyMerchantItem(row.index,private.quantity)
		end	
	end
end

function private:Show()
	local numMerchantItems = GetMerchantNumItems();
	
	local stData = {}	
	
	local name, texture, price, stackCount, numAvailable, isUsable, extendedCost
	local itemCostTexture, itemCostValue, itemCostLink
	
	for index = 1, numMerchantItems do
		local name, texture, price, stackCount, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(index)
		local itemLink = GetMerchantItemLink(index)
		
		local displayCost = ""
		local displayCostSeparator = "" 
		local displayQuantity = ""
		
		if price > 0 then
			displayCost = TSMAPI:FormatTextMoney(price)
			displayCostSeparator = ", "
		end
		
		if numAvailable > -1 then
			displayQuantity = format("%d",numAvailable)
		end
		
		if extendedCost then
			local costCount = GetMerchantItemCostInfo(index)
			for i = 1,costCount do								
				itemCostTexture, itemCostValue, itemCostLink = GetMerchantItemCostItem(index,i)				
				
				if itemCostTexture and itemCostValue then
					displayCost = format("%s%s%d |T%s:0|t",displayCost,displayCostSeparator,itemCostValue,itemCostTexture)
				else
					displayCost = format("%s%s%d", displayCost, displayCostSeparator,itemCostValue)
				end
												
				displayCostSeparator = ", "
			end			
		end
 					
		if itemLink then		
			tinsert(stData, { 
							cols = { 
								{ value = format("%s", itemLink) }, 
								{ value = displayCost }, 
								{ value = displayQuantity }
							}, 
							index = index,
							itemLink = itemLink,
							stackCount = stackCount
						})
		end
	end		
	
	private.frame.st:SetData(stData)	
	
end
