-- ------------------------------------------------------------------------------ --
--                           TradeSkillMaster_Vendoring                           --
--           http://www.curse.com/addons/wow/tradeskillmaster_vendoring           --
--                                                                                --
--             A TradeSkillMaster Addon (http://tradeskillmaster.com)             --
--    All Rights Reserved* - Detailed license information included with addon.    --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local MerchantTab = TSM:NewModule("MerchantTab", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendoring") -- loads the localization table

local private = {tabs = {}}

function MerchantTab:OnEnable()
	MerchantTab:RegisterEvent("MERCHANT_SHOW", function() TSMAPI:CreateTimeDelay(0, private.OnMerchantShow) end)
	MerchantTab:RegisterEvent("PLAYER_MONEY", function() private:PlayerMoney() end)	
	MerchantTab:RegisterEvent("CURRENCY_DISPLAY_UPDATE", function() private:PlayerMoney() end)
	
	MerchantTab:RawHook("MerchantFrame_UpdateBuybackInfo", function(...) if MerchantFrame.selectedTab == 2 then return MerchantTab.hooks.MerchantFrame_UpdateBuybackInfo(...) end end, true)
	MerchantTab:RawHook("MerchantFrame_UpdateMerchantInfo", function(...) if MerchantFrame.selectedTab == 1 then return MerchantTab.hooks.MerchantFrame_UpdateMerchantInfo(...) end end, true)
end

function private:PlayerMoney()	
	if private.frame then
		private:UpdateMoney()		
	end
end

function private:OnMerchantUpdate()
	
end

function private:UpdateMoney()
	private.frame.moneyText:SetMoney(GetMoney())
	
	local currencies = { GetMerchantCurrencies() };
	
	local currencyText = ""
	
	if #currencies > 0 then
		for index = 1, #currencies do
		
			local name, count, icon = GetCurrencyInfo(currencies[index]);
			currencyText = format("%s %d |T%s:0|t",currencyText,count, icon)
		end	
	end
	
	private.frame.currencyText:SetText(currencyText)
end

function private:OnMerchantShow()
	private.frame = private.frame or private:CreateMerchantTab()
	if TSM.db.global.defaultMerchantTab then
		for i=1, MerchantFrame.numTabs do
			if _G["MerchantFrameTab"..i].isTSMTab then
				_G["MerchantFrameTab"..i]:Click()
				break
			end
		end
	end

	local currentTab = PanelTemplates_GetSelectedTab(MerchantFrame)	
	_G["MerchantFrameTab"..currentTab]:Click()	
	
	private:UpdateMoney()
end

function private:CreateMerchantTab()
	local frame = CreateFrame("Frame", nil, MerchantFrame)
	TSMAPI.Design:SetFrameBackdropColor(frame)
	frame:Hide()
	frame:SetPoint("TOPLEFT")
	frame:SetPoint("BOTTOMRIGHT", 40, 0)	
	frame:SetHeight(10)
	frame:EnableMouse(true)
	
	local function OnTabClick(self)
		PanelTemplates_SetTab(MerchantFrame, self:GetID())
	
		MerchantFrameInset:Hide()
		MerchantFramePortraitFrame:Hide()
		MerchantFrameBg:Hide()
		if MerchantFrameText then MerchantFrameText:Hide() end
		MerchantFrameTitleBg:Hide()
		MerchantFrameTitleText:Hide()
		MerchantFrameCloseButton:Hide()
		
		MerchantPageText:Hide()
		MerchantFrameLootFilter:Hide()		
		MerchantMoneyFrame:Hide()
		MerchantMoneyInset:Hide()
		MerchantExtraCurrencyInset:Hide()
		MerchantExtraCurrencyBg:Hide()
		MerchantMoneyBg:Hide()
		
		if MerchantRepairItemButton then MerchantRepairItemButton:Hide() end
		if MerchantGuildBankRepairButton then MerchantGuildBankRepairButton:Hide() end
		if MerchantRepairAllButton then MerchantRepairAllButton:Hide() end
		if MerchantToken1 then MerchantToken1:Hide() end			
		
		for i = 1, 12 do		
		   local merchantItem = _G["MerchantItem"..i]	   
		   if merchantItem then merchantItem:Hide() end
		end
		
		if MerchantBuyBackItem then MerchantBuyBackItem:Hide() end
		
		MerchantNameText:Hide()
		MerchantFrameLeftBorder:Hide()
		MerchantFrameTopBorder:Hide()
		MerchantFrameRightBorder:Hide()
		MerchantFrameBottomBorder:Hide()
		MerchantFrameTopTileStreaks:Hide()
		MerchantFrameTopRightCorner:Hide()
		MerchantFrameBotLeftCorner:Hide()
		MerchantFrameBotRightCorner:Hide()
		MerchantNextPageButton:Hide()
		MerchantPrevPageButton:Hide()
		MerchantRepairText:Hide()
		MerchantFrameBottomLeftBorder:Hide()
		MerchantFrameBottomRightBorder:Hide()
		MerchantFrameBotLeftCorner:Hide()
		MerchantFrameBotRightCorner:Hide()
		MerchantFrameBtnCornerLeft:Hide()
		MerchantFrameBtnCornerRight:Hide()
		MerchantFrameButtonBottomBorder:Hide()
		
		BuybackBG:Hide()		
		
		private.frame:Show()
		
		if TSM.db.global.defaultPage == 1 then
			private.frame.buyBtn:Click()
		elseif TSM.db.global.defaultPage == 2 then
			private.frame.buybackBtn:Click()
		elseif TSM.db.global.defaultPage == 3 then
			private.frame.groupsBtn:Click()
		elseif TSM.db.global.defaultPage == 4 then
			private.frame.quickSellBtn:Click()
		end		
						
		-- Detect SellJunk
		if IsAddOnLoaded("SellJunk") then LibStub("AceAddon-3.0"):GetAddon("SellJunk").sellButton:Hide() end
		
	end
	
	local function OnOtherTabClick(a,b,c)
		if not private.frame then return end
		private.frame:Hide()
			
		MerchantNameText:Show()
		MerchantFrameLeftBorder:Show()
		MerchantFrameTopBorder:Show()
		MerchantFrameRightBorder:Show()
		MerchantFrameBottomBorder:Show()
		MerchantFrameTopTileStreaks:Show()
		MerchantFrameTopRightCorner:Show()
		MerchantFrameBotLeftCorner:Show()
		MerchantFrameBotRightCorner:Show()		
		MerchantFrameBottomLeftBorder:Show()
		MerchantFrameBottomRightBorder:Show()
		
		MerchantFrameLootFilter:Show()
		MerchantMoneyFrame:Show()
		MerchantMoneyInset:Show()
		MerchantMoneyBg:Show()
		MerchantExtraCurrencyInset:Show()
		MerchantExtraCurrencyBg:Show()
		MerchantPageText:Show()
				
		MerchantFrameInset:Show()
		MerchantFramePortrait:Show()
		MerchantFramePortraitFrame:Show()
		MerchantFrameBg:Show()
		if MerchantFrameText then MerchantFrameText:Show() end
		MerchantFrameTitleBg:Show()
		MerchantFrameTitleText:Show()
		MerchantFrameCloseButton:Show()
		MerchantNextPageButton:Show()
		MerchantPrevPageButton:Show()		
		MerchantRepairText:Show()
		BuybackBG:Show()	
		
		if MerchantToken1 then MerchantToken1:Show() end			
		
		for i = 1, 12 do		
		   local merchantItem = _G["MerchantItem"..i]		  
		   if merchantItem then merchantItem:Show() end
		end
		
		if MerchantBuyBackItem then MerchantBuyBackItem:Show() end		
		
		-- Detect SellJunk
		if IsAddOnLoaded("SellJunk") then LibStub("AceAddon-3.0"):GetAddon("SellJunk").sellButton:Show() end		
				
		MerchantFrame_Update()
	end
	
	MerchantFrameTab1:HookScript("OnClick", OnOtherTabClick)
	MerchantFrameTab2:HookScript("OnClick", OnOtherTabClick)

	local n = MerchantFrame.numTabs + 1
	local tab = CreateFrame("Button", "MerchantFrameTab"..n, MerchantFrame, "FriendsFrameTabTemplate")
	tab:Hide()
	tab:SetID(n)
	tab:SetText(TSMAPI.Design:GetInlineColor("link2").."TSM_Vendoring|r")
	tab:SetNormalFontObject(GameFontHighlightSmall)
	tab.isTSMTab = true
	tab:SetPoint("LEFT", _G["MerchantFrameTab"..n-1], "RIGHT", -16, 0)
	tab:Show()
	tab:SetScript("OnClick", OnTabClick)
	PanelTemplates_SetNumTabs(MerchantFrame, n)
	PanelTemplates_EnableTab(MerchantFrame, n)
	frame.tab = tab
	
	local iconFrame = CreateFrame("Frame", nil, frame)
	iconFrame:SetPoint("CENTER", frame, "TOPLEFT", 25, -25)
	iconFrame:SetHeight(80)
	iconFrame:SetWidth(80)
	local icon = iconFrame:CreateTexture(nil, "ARTWORK")
	icon:SetAllPoints()
	icon:SetTexture("Interface\\Addons\\TradeSkillMaster\\Media\\TSM_Icon_Big")
	local ag = iconFrame:CreateAnimationGroup()
	local spin = ag:CreateAnimation("Rotation")
	spin:SetOrder(1)
	spin:SetDuration(2)
	spin:SetDegrees(90)
	local spin = ag:CreateAnimation("Rotation")
	spin:SetOrder(2)
	spin:SetDuration(4)
	spin:SetDegrees(-180)
	local spin = ag:CreateAnimation("Rotation")
	spin:SetOrder(3)
	spin:SetDuration(2)
	spin:SetDegrees(90)
	ag:SetLooping("REPEAT")
	iconFrame:SetScript("OnEnter", function() ag:Play() end)
	iconFrame:SetScript("OnLeave", function() ag:Stop() end)
	
	local title = TSMAPI.GUI:CreateLabel(frame)
	title:SetPoint("TOPLEFT", 40, -5)
	title:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -5, -25)
	title:SetJustifyH("CENTER")
	title:SetJustifyV("CENTER")
	title:SetText("TSM_Vendoring - "..(strfind(TSM._version, "@") and "Dev" or TSM._version))
	
	local closeBtn = TSMAPI.GUI:CreateButton(frame, 19)
	closeBtn:SetPoint("TOPRIGHT", -5, -5)
	closeBtn:SetWidth(20)
	closeBtn:SetHeight(20)
	closeBtn:SetText("X")
	closeBtn:SetScript("OnClick", CloseMerchant)
	
	local line = TSMAPI.GUI:CreateVerticalLine(frame, 0)
	line:ClearAllPoints()
	line:SetPoint("TOPRIGHT", -30, -1)
	line:SetWidth(2)
	line:SetHeight(30)
	TSMAPI.GUI:CreateHorizontalLine(frame, -30)
	
	private:CreateTabs(frame)
	return frame
end

function private:CreateTabs(frame)
	
	local function OnButtonClick(self)
		frame.quickSellTab:Hide()
		frame.groupsTab:Hide()
		frame.buybackTab:Hide()
		frame.buyTab:Hide()
		
		frame.quickSellBtn:UnlockHighlight()
		frame.groupsBtn:UnlockHighlight()
		frame.buybackBtn:UnlockHighlight()
		frame.buyBtn:UnlockHighlight()
		self:LockHighlight()
	
		if self == frame.quickSellBtn then
			frame.quickSellTab:Show()
		elseif self == frame.groupsBtn then
			frame.groupsTab:Show()
		elseif self == frame.buyBtn then
			frame.buyTab:Show()
		elseif self == frame.buybackBtn then
			frame.buybackTab:Show()
		end
	end
	
	local button = TSMAPI.GUI:CreateButton(frame, 15)
	button:SetPoint("TOPLEFT", 70, -40)
	button:SetHeight(20)
	button:SetWidth(55)
	button:SetText(L["Buy"])
	button:SetScript("OnClick", OnButtonClick)
	frame.buyBtn = button	
	
	local button = TSMAPI.GUI:CreateButton(frame, 15)
	button:SetPoint("TOPLEFT", frame.buyBtn, "TOPRIGHT", 5, 0)
	button:SetHeight(20)
	button:SetWidth(55)
	button:SetText(L["Buyback"])
	button:SetScript("OnClick", OnButtonClick)
	frame.buybackBtn = button	

	local button = TSMAPI.GUI:CreateButton(frame, 15)
	button:SetPoint("TOPLEFT", frame.buybackBtn, "TOPRIGHT", 5, 0)
	button:SetHeight(20)
	button:SetWidth(80)
	button:SetText(L["TSM Groups"])
	button:SetScript("OnClick", OnButtonClick)
	frame.groupsBtn = button	
	
	local button = TSMAPI.GUI:CreateButton(frame, 15)
	button:SetPoint("TOPLEFT", frame.groupsBtn, "TOPRIGHT", 5, 0)
	button:SetHeight(20)
	button:SetWidth(70)
	button:SetText(L["Quick Sell"])
	button:SetScript("OnClick", OnButtonClick)
	frame.quickSellBtn = button		
	
	TSMAPI.GUI:CreateHorizontalLine(frame, -70)
	
	local moneyText = TSMAPI.GUI:CreateLabel(frame)
	moneyText:SetJustifyH("CENTER")
	moneyText:SetJustifyV("CENTER")
	moneyText:SetPoint("CENTER", frame, "BOTTOMLEFT", 85, 17)
	TSMAPI.Design:SetIconRegionColor(moneyText)
	moneyText.SetMoney = function(self, money)
		self:SetText(TSMAPI:FormatTextMoneyIcon(money))
	end
	frame.moneyText = moneyText
	
	local currencyText = TSMAPI.GUI:CreateLabel(frame)
	currencyText:SetPoint("TOPLEFT", frame.moneyText, "TOPRIGHT", 5, 0)
	frame.currencyText = currencyText
	
	local content = CreateFrame("Frame", nil, frame)
	content:SetPoint("TOPLEFT", 0, -30)
	content:SetPoint("BOTTOMRIGHT")
		
	local repairBtn = TSMAPI.GUI:CreateButton(frame, 15)
	
	--repairBtn.icon = widget:CreateTexture()
	--repairBtn.icon:SetAllPoints()
	--repairBtn.icon:SetTexture(TEXTURE)
	frame.repairBtn = repairBtn
	
	local repairAllBtn = TSMAPI.GUI:CreateButton(frame, 15)
	--repairAllBtn.icon = widget:CreateTexture()
	--repairAllBtn.icon:SetAllPoints()
	--repairAllBtn.icon:SetTexture(TEXTURE)
	frame.repairAllBtn = repairAllBtn	
	
	local repairAllBankBtn = TSMAPI.GUI:CreateButton(frame, 15)
	repairAllBankBtn:SetNormalTexture("Interface\MerchantFrame\UI-Merchant-RepairIcons")
	--repairAllBankBtn.icon = widget:CreateTexture()
	--repairAllBankBtn.icon:SetAllPoints()
	--repairAllBankBtn.icon:SetTexture(TEXTURE)
	frame.repairAllBankBtn = repairAllBankBtn	
	
	--:SetNormalTexture(...)/:SetPushedTexture(...)/:SetDisabledTexture(...)/:SetHighlightTexture(...) instead on the button

	frame.quickSellTab = TSM.QuickSell:CreateTab(content)
	frame.quickSellTab:Hide()	
	
	frame.buyTab = TSM.Buy:CreateTab(content)
	frame.buyTab:Hide()
	
	frame.groupsTab = TSM.Groups:CreateTab(content)
	frame.groupsTab:Hide()
	
	frame.buybackTab = TSM.Buyback:CreateTab(content)
	frame.buybackTab:Hide()	
end