-- ------------------------------------------------------------------------------ --
--                           TradeSkillMaster_Vendoring                           --
--           http://www.curse.com/addons/wow/tradeskillmaster_vendoring           --
--                                                                                --
--             A TradeSkillMaster Addon (http://tradeskillmaster.com)             --
--    All Rights Reserved* - Detailed license information included with addon.    --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local QuickSell = TSM:NewModule("QuickSell", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendoring") -- loads the localization table

local private = { allowTimerStart = true, ignore = {}, sellAmount = 0, sellIndex = 0, sellQueue = {}, sellInfo = {}, autoSelling = false, updateFunction = nil, completedFunction = nil }

function QuickSell:OnEnable()

end

function QuickSell:CreateTab(parent)	
	local frame = CreateFrame("Frame", nil, parent)
	frame:Hide()
	frame:SetAllPoints()
	frame:SetScript("OnShow", function() private.TabShow() end)

	local label = TSMAPI.GUI:CreateLabel(frame, "small")
	label:SetPoint("TOPLEFT", 5, -5)
	label:SetPoint("TOPRIGHT", -5, -5)
	label:SetHeight(15)
	label:SetJustifyH("CENTER")
	label:SetJustifyV("CENTER")
	
	local stContainer = CreateFrame("Frame", nil, frame)
	stContainer:SetPoint("TOPLEFT", 5, -45)
	stContainer:SetPoint("BOTTOMRIGHT", -15, 100)
	TSMAPI.Design:SetFrameColor(stContainer)

	local handlers = {
		OnClick = function(_, data, self, button)
			if not data then return end
						
			if button == "RightButton" then
				if IsShiftKeyDown() then
					TSM.db.global.ignore[data.itemString] = true
					TSM:Printf(L["Ignoring all %s permanently. You can undo this in the Vendoring options."], data.itemLink)
					TSM.Options:UpdateIgnoreST()
				else
					private.ignore[data.itemString] = true
					TSM:Printf(L["Ignoring all %s this session (until your UI is reloaded)."], data.itemLink)
				end
				private:MerchantUpdate()
			end
			
			if button == "LeftButton" then							
				if IsShiftKeyDown() then
					UseContainerItem(data.bag,data.slot)
										
					private.updateFunction = function() private:MerchantUpdate() end
					TSMAPI:RegisterForBagChange(private.updateFunction)
				end										
			end
		end,
		OnEnter = function(_, data, self)
			if not data then return end
			local color = TSMAPI.Design:GetInlineColor("link")
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			TSMAPI:SafeTooltipLink(data.itemLink)
			GameTooltip:AddLine(format(L["%sRight-Click|r to ignore this item for this session. Hold %sshift|r to ignore permanently. You can remove items from permanent ignore in the Vendoring options."], color, color), 1, 1, 1, 1, true)
			GameTooltip:Show()
		end,
		OnLeave = function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
			BattlePetTooltip:Hide()
		end
	}
	
	
	local stCols = {
		{
			name = L["Item"],
			width = 0.4,
		},
		{
			name = L["Vendor"],
			width = 0.2,
		},
		{
			name = L["Potential"],
			width = 0.2
		},
		{
			name = L["Exp"],
			width = 0.1
		}
	}	
	
	local checkBox = TSMAPI.GUI:CreateCheckBox(frame)
	checkBox:SetPoint("BOTTOMLEFT", 5, 65)
	checkBox:SetPoint("BOTTOMRIGHT", -5, 65)
	checkBox:SetHeight(30)
	checkBox:SetLabel(L["Hide Grouped Items"])
	checkBox:SetValue(TSM.db.global.qsHideGrouped)
	checkBox:SetCallback("OnValueChanged", function(_, _, value)
			TSM.db.global.qsHideGrouped = value
			private:MerchantUpdate()
		end)	

	local st = TSMAPI:CreateScrollingTable(stContainer, stCols, handlers)
	st:SetData({})
	st:DisableSelection(true)
	frame.st = st
			
	local label = TSMAPI.GUI:CreateLabel(frame, "normal")
	label:SetPoint("BOTTOMLEFT", 5, 25)
	label:SetHeight(20)
	label:SetJustifyH("LEFT")
	label:SetJustifyV("CENTER")
	label:SetText("Sell Only:")
	
	local btn = TSMAPI.GUI:CreateButton(frame, 18)
	btn:SetPoint("BOTTOMLEFT", 5, 50)
	btn:SetPoint("BOTTOMRIGHT", -5, 50)
	btn:SetHeight(20)
	btn:SetText("Sell All")
	btn:SetScript("OnClick", function() private:StartAutoSelling("all") end)
	frame.sellAllBtn = btn
	
	local btnWidth = (frame:GetWidth() - label:GetWidth() - 25) / 4	
	
	local btn = TSMAPI.GUI:CreateButton(frame, 18)
	btn:SetPoint("BOTTOMLEFT", label, "BOTTOMRIGHT", 5, 0)
	btn:SetWidth(btnWidth)
	btn:SetHeight(20)
	btn:SetText(L["Trash"])
	btn:SetScript("OnClick", function() private:StartAutoSelling("trash") end)
	frame.sellTrashBtn = btn
		
	frame.EnableButtons = function(self)
		self.sellAllBtn:Enable()		
		self.sellTrashBtn:Enable()
		self.buttonsEnabled = true
	end
	frame.DisableButtons = function(self)
		self.sellAllBtn:Disable()
		self.sellTrashBtn:Disable()
		self.buttonsEnabled = nil
	end
	frame.buttonsEnabled = true

	private.frame = frame
	return frame
end

function private:StartAutoSelling(mode)	
	private.moneyCollected = 0	
	private.frame:DisableButtons()	
	
	private.sellQueue = {}
	
	private.sellIndex = 0
	private.sellAmount = 0
	private.autoSelling = true
	
	for j, info in ipairs(private.sellInfo) do		
		if(private.sellAmount < TSM.db.global.qsNumToSell) then
			if mode == "all" then
				private.sellQueue[private.sellAmount] = { bag = info.bag, slot = info.slot, itemString = info.itemString, itemLink = info.itemLink }
				private.sellAmount = private.sellAmount + 1		
			elseif mode == "trash" and info.isTrash then
				private.sellQueue[private.sellAmount] = { bag = info.bag, slot = info.slot, itemString = info.itemString, itemLink = info.itemLink }
				private.sellAmount = private.sellAmount + 1			
			end			
		end
	end
		
	private:AutoSell()
end

function private:AutoSell()
	for i = 0, private.sellAmount-1 do		
		UseContainerItem(private.sellQueue[i].bag,private.sellQueue[i].slot)				
		private.sellIndex = i
	end
	
	private.updateFunction = function() private:CompleteAutoSell() end
	TSMAPI:RegisterForBagChange(private.updateFunction)
	
	private.autoSelling = false
end

function private:TabShow()
	private:MerchantUpdate()
end

function private:CompleteAutoSell()
	private:MerchantUpdate()
	
	if not private.autoSelling then
		private.frame:EnableButtons()
	end		
end

function private:MerchantUpdate()		
	
	if not private.frame then
		return
	end
	
	if not private.frame.st then
		return
	end
	
	private.sellInfo = {}
		
	for bag,slot,itemString,quantity in TSMAPI:GetBagIterator(nil,true) do		
		local values = private:ShouldSell(bag,slot,quantity)
										
		if values then		
			table.insert(private.sellInfo,values)
		end			
	end
		
	local stData = {}	
	for j, info in ipairs(private.sellInfo) do	
		tinsert(stData, { cols = { { value = format("%s", info.itemLink) }, 
								   { value = TSMAPI:FormatTextMoney(info.vendorValue) }, 
								   { value = TSMAPI:FormatTextMoney(info.potentialValue) },
								   { value = format("%d", info.expired) },
								}, index = j, itemString = info.itemString, itemLink = info.itemLink, bag = info.bag, slot = info.slot, isTrash = info.isTrash, isExpired = info.isExpired })
	end
	
	private.frame.st:SetData(stData)
end

function private:ShouldSell(bag,slot,quantity)
	local itemLink = GetContainerItemLink(bag,slot)
	local itemString = TSMAPI:GetItemString(itemLink)
	
	if itemLink then				
		local vendorValue = TSMAPI:GetItemValue(itemLink,'VendorSell') or 0	
		
		if vendorValue == 0 then
			return
		end		
		
		if TSM.db.global.qsHideGrouped and TSMAPI:GetGroupPath(itemString) then
			return
		end	
		
		if private.ignore[itemString] or TSM.db.global.ignore[itemString] then
			return
		end
			
		local grey = string.find(itemLink,"|cff9d9d9d")
					
		if grey then				
			return { 
				itemString = itemString, itemLink = itemLink, 
				Sell = true, 
				vendorValue = vendorValue, 
				potentialValue = vendorValue,
				expired = 0,
				bag = bag,
				slot = slot,
				isTrash = true,
				isExpired = false,
			}
		end
				
		if not TSMAPI:IsSoulbound(bag, slot) then		
					
			local cancelledNum, expiredNum, totalFailed = TSMAPI:ModuleAPI("Accounting", "getAuctionStatsSinceLastSale", itemString)
			
			local qsExpired = TSM.db.global.qsExpired or 0
			local qsMarketValue = TSM.db.global.qsMarketValue
			local qsDisenchantValue = TSM.db.global.qsDisenchantValue
			local qsMaxMarketValue = TSM.db.global.qsMaxMarketValue
			local qsMaxDisenchantValue = TSM.db.global.qsMaxDisenchantValue
			local alwaysSell = false
			
			if not cancelledNum or not expiredNum then
				expiredNum = 0
			end						
		
			local operation = (TSMAPI:GetItemOperation(itemString, "Vendoring") or {})[1]
														
			if operation then					
				local opSettings = TSM.operations[operation]
				
				if opSettings then
					if opSettings.qsDoNotSell then
						return
					end
					
					if opSettings.qsAlwaysSell then
						alwaysSell = true
					end
					
					if opSettings.qsExpired then
						qsExpired = opSettings.qsExpired
					end
					
					if opSettings.qsMarketValue then
						qsMarketValue = opSettings.qsMarketValue
					end
					
					if opSettings.qsDisenchantValue then
						qsDisenchantValue = opSettings.qsDisenchantValue
					end
					
					if opSettings.qsMaxMarketValue then
						qsMaxMarketValue = opSettings.qsMaxMarketValue
					end
					
					if opSettings.qsMaxDisenchantValue then
						qsMaxMarketValue = opSettings.qsMaxMarketValue
					end									
				end							
			end				
			
			local disenchantValue = private:GetPrice(qsDisenchantValue,itemLink) or 0
			local marketValue = private:GetPrice(qsMarketValue,itemLink) or 0
			
			local maxMarketValue = private:GetPrice(qsMaxMarketValue,itemLink) or 0
			local maxDisenchantValue = private:GetPrice(qsMaxDisenchantValue,itemLink) or 0			
											
			if qsExpired > 0 and expiredNum >= qsExpired then
				return { 
					itemString = itemString, itemLink = itemLink, 
					Sell = true, 
					vendorValue = vendorValue, 
					potentialValue = max(disenchantValue,marketValue), 
					expired = expiredNum,
					bag = bag,
					slot = slot,
					isTrash = false,
					isExpired = true
				}					
			end
						
			if not alwaysSell then
				if disenchantValue > maxDisenchantValue then
					return
				end
				
				if marketValue > maxMarketValue then
					return
				end						
			end
									
			return { 
				itemString = itemString, itemLink = itemLink, 
				Sell = true, 
				vendorValue = vendorValue, 
				potentialValue = max(disenchantValue,marketValue), 
				expired = expiredNum,
				bag = bag,
				slot = slot,
				isTrash = false,
				isExpired = false,
			}		
		end
	end
	
	return
end

function private:GetPrice(operationPrice, itemString)
	local price, err
	if type(operationPrice) == "number" then
		price = operationPrice
	elseif type(operationPrice) == "string" then
		local func, parseErr = TSMAPI:ParseCustomPrice(operationPrice)
		err = parseErr
		price = func and func(itemString)
	end
	return price ~= 0 and price or nil, err
end